/*
 * Copyright (c) 2008 Hagen Paul Pfeifer <hagen@jauu.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <iostream>
#include <fstream>
#include <string>

#include "ns3/simulator-module.h"
#include "ns3/core-module.h"
#include "ns3/common-module.h"
#include "ns3/node-module.h"
#include "ns3/helper-module.h"

using namespace ns3;

#define	NUMER_ITERATIONS 6
#define	WORKERS (Worker::GetNumCores ())

#define	COMMUNICATION_STOP 100.0

using namespace ns3;

void adhoc_scenario(uint32_t id)
{
  uint32_t i;

  for (i =  Worker::ChunkStart (id, WORKERS, NUMER_ITERATIONS);
       i <= Worker::ChunkEnd   (id, WORKERS, NUMER_ITERATIONS);
	   i++)
  {
    std::cerr << "Worker " << id << " process job #"
	          << i << std::endl;

    NodeContainer c;
    c.Create (5);
    NodeContainer n02 = NodeContainer (c.Get(0), c.Get (2));
    NodeContainer n12 = NodeContainer (c.Get(1), c.Get (2));
    NodeContainer n32 = NodeContainer (c.Get(3), c.Get (2));
    NodeContainer n34 = NodeContainer (c.Get (3), c.Get (4));

    InternetStackHelper internet;
    internet.Install (c);

    // We create the channels first without any IP addressing information
    PointToPointHelper p2p;
    p2p.SetDeviceAttribute ("DataRate", StringValue ("5Mbps"));
    p2p.SetChannelAttribute ("Delay", StringValue ("2ms"));
    NetDeviceContainer nd02 = p2p.Install (n02);
    NetDeviceContainer nd12 = p2p.Install (n12);
    p2p.SetDeviceAttribute ("DataRate", StringValue ("1500kbps"));
    p2p.SetChannelAttribute ("Delay", StringValue ("10ms"));
    NetDeviceContainer nd32 = p2p.Install (n32);
    NetDeviceContainer nd34 = p2p.Install (n34);

    Ipv4AddressHelper ipv4;
    ipv4.SetBase ("10.1.1.0", "255.255.255.0");
    Ipv4InterfaceContainer i02 = ipv4.Assign (nd02);

    ipv4.SetBase ("10.1.2.0", "255.255.255.0");
    Ipv4InterfaceContainer i12 = ipv4.Assign (nd12);

    ipv4.SetBase ("10.1.3.0", "255.255.255.0");
    Ipv4InterfaceContainer i32 = ipv4.Assign (nd32);

    ipv4.SetBase ("10.1.4.0", "255.255.255.0");
    Ipv4InterfaceContainer i34 = ipv4.Assign (nd34);

    OlsrHelper olsr;
    olsr.InstallAll ();

    uint16_t port = 9;   // Discard port (RFC 863)

    OnOffHelper onoff ("ns3::UdpSocketFactory",
                       InetSocketAddress (i34.GetAddress (1), port));
    onoff.SetAttribute ("OnTime", RandomVariableValue (ConstantVariable (1)));
    onoff.SetAttribute ("OffTime", RandomVariableValue (ConstantVariable (0)));

    ApplicationContainer apps = onoff.Install (c.Get (0));
    apps.Start (Seconds (1.0));
    apps.Stop (Seconds (COMMUNICATION_STOP));

    // Create a packet sink to receive these packets
    PacketSinkHelper sink ("ns3::UdpSocketFactory",
                           InetSocketAddress (Ipv4Address::GetAny (), port));

    apps = sink.Install (c.Get (3));
    apps.Start (Seconds (1.0));
    apps.Stop (Seconds (COMMUNICATION_STOP));

    // Create a similar flow from n3 to n1, starting at time 1.1 seconds
    onoff.SetAttribute ("Remote",
                        AddressValue (InetSocketAddress (i12.GetAddress (0), port)));
    apps = onoff.Install (c.Get (3));
    apps.Start (Seconds (1.1));
    apps.Stop (Seconds (COMMUNICATION_STOP));

    // Create a packet sink to receive these packets
    apps = sink.Install (c.Get (1));
    apps.Start (Seconds (1.1));
    apps.Stop (Seconds (COMMUNICATION_STOP));

    std::ofstream ascii;
    char outstr[128];
    sprintf(outstr, "simple-point-to-point-olsr-%d.tr", i);
    ascii.open (outstr);
    PointToPointHelper::EnableAsciiAll (ascii);

    Simulator::Stop (Seconds (COMMUNICATION_STOP));

    Simulator::Run ();
    Simulator::Destroy ();
  }

  return;
}


int main (int argc, char *argv[])
{

  Worker worker;

  // Inform the worker to spawn n subworkers
  // This number should be identical with the
  // number of cores in your machine:
  //    Single Core - 1
  //	Dual Core   - 2
  //	Quad Core   - 4
  //	...
  // GetNumCores() try to determine this automatically
  // via sysconf(3) or will fall back to 1 (an acceptable default)
  worker.SetWorkers (WORKERS);
  worker.AddCallback (MakeCallback (adhoc_scenario));

  worker.Run ();

  return 0;
}
