/*
 * Copyright (c) 2008 Hagen Paul Pfeifer <hagen@jauu.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "worker.h"
#include "log.h"
#include "config.h"
#include "global-value.h"
#include "type-id.h"
#include "string.h"

#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>

NS_LOG_COMPONENT_DEFINE ("Worker");

namespace ns3 {

Worker::~Worker ()
{
}

void
Worker::AddCallback (Callback<void, uint32_t> callback)
{
  m_callback = callback;
}

void
Worker::SetWorkers (uint32_t workers)
{
  m_workers = workers;
}


void
Worker::Run () const
{
  uint32_t i;

  if (m_workers < 1) // FIXME: add some warnings
    return;

  pid_t child[m_workers];

  for (i = 0; i < m_workers; i++) {
    child[i] = fork();

    switch (child[i]) {
      case -1:
        perror("fork():");
        break;
      case 0: /* child */
        m_callback(i);
        exit(0);
        break;
      default: /* parent */
        sched_yield();
        break;
    }
  }

  /* reap death childs */
  for (i = 0; i < m_workers; i++) {
    pid_t child; int status;
    child = waitpid(-1, &status, 0);
  }
}


} // namespace ns3

