/*
 * Copyright (c) 2008 Hagen Paul Pfeifer <hagen@jauu.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#ifndef WORKER_H
#define WORKER_H

#include <string>
#include <sstream>
#include <list>

#include "ns3/callback.h"

namespace ns3 {

class Worker
{

public:
  ~Worker ();

  void AddCallback (Callback<void, uint32_t> callback);
  void SetWorkers (uint32_t no_workers);
  void Run () const;

  static inline uint32_t ChunkStart (uint32_t index, uint32_t worker, uint32_t max)
  {
	// FIXME handle the case sanely where max < worker
    return (max / worker) * index;
  }

  static inline uint32_t ChunkEnd (uint32_t index, uint32_t worker, uint32_t max)
  {
    uint32_t chuck_end;

    uint32_t res = max / worker;
    uint32_t mod = max % worker;

    chuck_end = (res * index) + res - 1;
    // add remaining for the last chunk
    if (chuck_end + mod == max - 1)
      chuck_end = max - 1;

    return chuck_end;
  }


  static uint32_t GetNumCores ()
  {
    int32_t cores;

    cores = sysconf(_SC_NPROCESSORS_ONLN);

    // in the case of a sysconf error
    // we return 1. This should at least
    // guarantee a acceptable default.
    if (cores < 1)
	    return 1;

    return cores;
  }

private:
  uint32_t m_workers;
  Callback<void, uint32_t> m_callback;

};

} // namespace ns3


#endif /* WORKER_H */
